<?php
namespace Ak\Container\AbstractFactory;

use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Config;
use Laminas\ServiceManager\Factory\AbstractFactoryInterface;
use Laminas\ServiceManager\PluginManagerInterface;

/**
 * Kreira instance plugin managera koji su konfigurirani u glavnoj konfiguraciji.
 *
 * @package Ak\Container\AbstractFactory
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/app-container
 */
final class PluginManagerAbstractFactory implements AbstractFactoryInterface
{
    /**
     * Da li možemo napraviti instancu traženog plugin managera?
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     *
     * @return bool
     */
    public function canCreate(ContainerInterface $container, $requestedName)
    {
        return is_subclass_of($requestedName, PluginManagerInterface::class);
    }

    /**
     * Create an object
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return PluginManagerInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $pluginManager = new $requestedName($container);
        $configKeyConstant = sprintf('%s::CONFIG_KEY', $requestedName);

        if (defined($configKeyConstant) && $container->has('config')) {
            $config = $container->get('config');
            $configKey = constant($configKeyConstant);

            (new Config($config[$configKey] ?? []))->configureServiceManager($pluginManager);
        }

        return $pluginManager;
    }
}
