<?php
namespace AkTest\Container\AbstractFactory;

use Ak\Container\AbstractFactory\PluginManagerAbstractFactory;
use AkTest\Container\Mock;
use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Factory\InvokableFactory;
use Laminas\ServiceManager\ServiceManager;
use PHPUnit\Framework\TestCase;

/**
 * Class PluginManagerAbstractFactoryTest
 *
 * @package AkTest\Container\AbstractFactory
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/service-manager
 */
final class PluginManagerAbstractFactoryTest extends TestCase
{
    private function getMockContainer(array $config = []): ContainerInterface
    {
        $container = new ServiceManager($config['dependencies'] ?? []);
        $container->setService('config', $config);

        return $container;
    }

    public function testCanFigureOutIfItCanCreateAManager()
    {
        $container = $this->getMockContainer();
        $factory = new PluginManagerAbstractFactory();

        $this->assertFalse($factory->canCreate($container, Mock\PluginManagerInvalidInterface::class));
        $this->assertTrue($factory->canCreate($container, Mock\PluginManager::class));
    }

    public function testCanCreateAConfiguredPluginManager()
    {
        $container = $this->getMockContainer([
            'test_plugins' => [
                'aliases' => [
                    'plugin' => Mock\Plugin::class,
                ],
                'factories' => [
                    Mock\Plugin::class => InvokableFactory::class,
                ],
            ],
        ]);

        $factory = new PluginManagerAbstractFactory();
        $manager = $factory($container, Mock\PluginManager::class);

        $this->assertTrue($manager->has('plugin'));
        $this->assertInstanceOf(Mock\Plugin::class, $manager->get('plugin'));
    }

    public function testCanBeUsedAsAbstractFactory()
    {
        $container = $this->getMockContainer([
            'dependencies' => [
                'abstract_factories' => [
                    PluginManagerAbstractFactory::class,
                ],
            ],
        ]);

        $this->assertTrue($container->has(Mock\PluginManager::class));
        $this->assertInstanceOf(Mock\PluginManager::class, $container->get(Mock\PluginManager::class));
    }

    public function testCanBeUsedAsRegularFactory()
    {
        $container = $this->getMockContainer([
            'dependencies' => [
                'factories' => [
                    Mock\PluginManager::class => PluginManagerAbstractFactory::class,
                ],
            ],
        ]);

        $this->assertTrue($container->has(Mock\PluginManager::class));
        $this->assertInstanceOf(Mock\PluginManager::class, $container->get(Mock\PluginManager::class));
    }
}
